/* ========================================================================
 * PlantUML : a free UML diagram generator
 * ========================================================================
 *
 * (C) Copyright 2009-2020, Arnaud Roques
 *
 * Project Info:  https://plantuml.com
 * 
 * If you like this project or if you find it useful, you can support us at:
 * 
 * https://plantuml.com/patreon (only 1$ per month!)
 * https://plantuml.com/paypal
 * 
 * This file is part of PlantUML.
 *
 * PlantUML is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * PlantUML distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 *
 * Original Author:  Arnaud Roques
 */
package ext.plantuml.com.ctreber.acearth.gui;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.OutputStream;

import ext.plantuml.com.ctreber.acearth.renderer.RenderTarget;
import net.sourceforge.plantuml.security.ImageIO;

/**
 * <p>
 * Swing compatible drawing surface for images and graphics.
 * 
 * <p>
 * &copy; 2002 Christian Treber, ct@ctreber.com
 * 
 * @author Christian Treber, ct@ctreber.com
 * 
 */
public class PixelCanvas implements RenderTarget {
	final private int fImageWidth;
	final private int fImageHeight;
	final private BufferedImage fEarthImage2;

	/**
	 * <p>
	 * Construct a canvas of the specified size.
	 * 
	 * @param pWidth  Width
	 * @param pHeight Height
	 */
	public PixelCanvas(int pWidth, int pHeight) {
		fImageWidth = pWidth;
		fImageHeight = pHeight;
		fEarthImage2 = new BufferedImage(fImageWidth, fImageHeight, BufferedImage.TYPE_INT_RGB);
	}

	public Graphics2D getGraphics2D() {
		return fEarthImage2.createGraphics();
	}

	public void setPixel(int pX, int pY, int pA, int pR, int pG, int pB) {
		setPixel(pX, pY, new Color(pR, pG, pB, pA));
	}

	public void setPixel(int pX, int pY, Color pColor) {
		fEarthImage2.setRGB(pX, pY, pColor.getRGB());
	}

	public int getImageWidth() {
		return fImageWidth;
	}

	public int getImageHeight() {
		return fImageHeight;
	}

	public void saveToImage(OutputStream os) throws IOException {
		ImageIO.write(fEarthImage2, "png", os);
	}

}
