/* ========================================================================
 * PlantUML : a free UML diagram generator
 * ========================================================================
 *
 * (C) Copyright 2009-2020, Arnaud Roques
 *
 * Project Info:  https://plantuml.com
 * 
 * If you like this project or if you find it useful, you can support us at:
 * 
 * https://plantuml.com/patreon (only 1$ per month!)
 * https://plantuml.com/paypal
 * 
 * This file is part of PlantUML.
 *
 * PlantUML is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * PlantUML distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 *
 * Original Author:  Arnaud Roques
 */
package ext.plantuml.com.ctreber.acearth.projection;

import ext.plantuml.com.ctreber.acearth.util.Coordinate;
import ext.plantuml.com.ctreber.acearth.util.Point3D;

/**
 * <p>Cylindrical projection. Show Earth flatly spread out on rectangle.
 *
 * <p>&copy; 2002 Christian Treber, ct@ctreber.com
 * @author Christian Treber, ct@ctreber.com
 *
 */
public class ProjectionCyl extends Projection
{
  /**
   * <p>All of Earth is visible.
   *
   * @param pPoint
   * @return
   */
  public boolean isVisible(Point3D pPoint)
  {
    return true;
  }

  public Coordinate getLocation(int pX, int pY)
  {
    final Coordinate lRaw = new Coordinate(Math.atan(inverseFinalizeY(pY)),
      inverseFinalizeX(pX));

    return rotateReverse(lRaw.getPoint3DRads()).getCoordinate();
  }

  /**
   * <p>The scale is set so that a value of
   * 2PI gets mapped to the full image width times the magnification.
   * But see ProjectionOrtho.setScale().
   */
  protected void setScale()
  {
    // Makes 2PI come out as full image width
    fScale = fViewMagnification * fImageWidth / (2 * Math.PI);
  }

  /**
   * @return Longitude (-PI to PI), linearly on x axis.
   */
  public double projectX(double pX, double pZ)
  {
    return Math.atan2(pX, pZ);
  }

  public double inverseProjectX(double pX)
  {
    return Math.sin(pX);
  }

  /**
   * @return Latitude (-PI/2 to PI/2), projected from center of Earth on
   * y axis with a linear scale.
   */
  public double projectY(double pY)
  {
    return (pY >= 0.9999999999) ? 1e6 :
      (pY <= -0.9999999999) ? -1e6 : Math.tan(Math.asin(pY));
  }

  public double inverseProjectY(double y)
  {
    return Math.sin(Math.atan(y));
  }
}
