/* ========================================================================
 * PlantUML : a free UML diagram generator
 * ========================================================================
 *
 * (C) Copyright 2009-2020, Arnaud Roques
 *
 * Project Info:  https://plantuml.com
 * 
 * If you like this project or if you find it useful, you can support us at:
 * 
 * https://plantuml.com/patreon (only 1$ per month!)
 * https://plantuml.com/paypal
 * 
 * This file is part of PlantUML.
 *
 * PlantUML is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * PlantUML distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 *
 * Original Author:  Arnaud Roques
 */
package ext.plantuml.com.ctreber.acearth.projection;

import ext.plantuml.com.ctreber.acearth.util.Coordinate;
import ext.plantuml.com.ctreber.acearth.util.Point3D;


/**
 * <p>Orthographic projection (show Earth as a ball).
 *
 * <p>&copy; 2002 Christian Treber, ct@ctreber.com
 * @author Christian Treber, ct@ctreber.com
 *
 */
public class ProjectionOrtho extends Projection
{
  /**
   * <p>All of Earth is visible.
   *
   * @param pPoint
   * @return
   */
  public boolean isVisible(Point3D pPoint)
  {
    return pPoint.getZ() >= 0;
  }

  public Coordinate getLocation(int pX, int pY)
  {
    final double lX = inverseFinalizeX(pX);
    final double lY = inverseFinalizeY(pY);
    final double lZ = Math.sqrt(1 - lX * lX - lY * lY);
    final Point3D lP = new Point3D(lX, lY, lZ);

    return rotateReverse(lP).getCoordinate();
  }

  /**
   * @return Longitude, not in rad but from -1 to 1.
   */
  public double projectX(double pX, double pZ)
  {
    return pX;
  }

  public double inverseProjectX(double pX)
  {
    return pX;
  }

  /**
   * @return Latitude, not in rad but from -1 to 1.
   */
  public double projectY(double pY)
  {
    return pY;
  }

  public double inverseProjectY(double pY)
  {
    return pY;
  }

  /**
   * <p>The scale is not from -PI to PI but from -1 to 1 in this case
   * (the range of x, y, z of the points).
   */
  protected void setScale()
  {
    fScale = Math.min(fImageHeight, fImageWidth) * fViewMagnification * 0.99 / 2;
  }
}
