/* ========================================================================
 * PlantUML : a free UML diagram generator
 * ========================================================================
 *
 * (C) Copyright 2009-2020, Arnaud Roques
 *
 * Project Info:  https://plantuml.com
 * 
 * If you like this project or if you find it useful, you can support us at:
 * 
 * https://plantuml.com/patreon (only 1$ per month!)
 * https://plantuml.com/paypal
 * 
 * This file is part of PlantUML.
 *
 * PlantUML is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * PlantUML distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 *
 * Original Author:  Arnaud Roques
 */
package ext.plantuml.com.ctreber.acearth.renderer;

import ext.plantuml.com.ctreber.acearth.scanbit.BitGeneratorMap;
import ext.plantuml.com.ctreber.acearth.scandot.ScanDot;

/**
 * <p>Renders a row of ScanDots to pixel types.</p>
 *
 * <p>&copy; 2002 Christian Treber, ct@ctreber.com (Nov 11, 2002)</p>
 * @author Christian Treber, ct@ctreber.com
 *
 */
public class RowTypeRendererScanDot implements RowTypeRenderer
{
  private int fScanDotIndex;
  private ScanDot[] fScanDots;

  public void startNewRun()
  {
    fScanDotIndex = 0;
  }

  public void getPixelTypes(int pRowNo, int[] pPixelTypes)
  {
    // For all ScanDots in specified row...
    while((fScanDotIndex < fScanDots.length) &&
      (fScanDots[fScanDotIndex].getY() == pRowNo))
    {
      ScanDot lDot = fScanDots[fScanDotIndex];

      if(lDot.getType() == ScanDot.DotTypeStar)
      {
        if(pPixelTypes[lDot.getX()] == BitGeneratorMap.PixTypeSpace)
        {
          // Stars get only painted on Space.
          pPixelTypes[lDot.getX()] = BitGeneratorMap.PixTypeStar;
        }
      } else
      {
        // The only other type for a dot (so far) is "grid".
        switch(pPixelTypes[lDot.getX()])
        {
          case BitGeneratorMap.PixTypeLand:
            pPixelTypes[lDot.getX()] = BitGeneratorMap.PixTypeGridLand;
            break;

          case BitGeneratorMap.PixTypeWater:
            pPixelTypes[lDot.getX()] = BitGeneratorMap.PixTypeGridWater;
            break;
        }
      }
      fScanDotIndex++;
    }
  }

  public void setScanDots(ScanDot[] pScanDots)
  {
    fScanDots = pScanDots;
  }
}
