/* ========================================================================
 * PlantUML : a free UML diagram generator
 * ========================================================================
 *
 * (C) Copyright 2009-2020, Arnaud Roques
 *
 * Project Info:  https://plantuml.com
 * 
 * If you like this project or if you find it useful, you can support us at:
 * 
 * https://plantuml.com/patreon (only 1$ per month!)
 * https://plantuml.com/paypal
 * 
 * This file is part of PlantUML.
 *
 * PlantUML is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * PlantUML distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 *
 * Original Author:  Arnaud Roques
 */
package ext.plantuml.com.ctreber.acearth.scandot;

import ext.plantuml.com.ctreber.acearth.projection.Projection;
import ext.plantuml.com.ctreber.acearth.util.Coordinate;
import ext.plantuml.com.ctreber.acearth.util.Point2D;
import ext.plantuml.com.ctreber.acearth.util.Point3D;

/**
 * <p>Generate latitude and longitude grid as dots.
 *
 * <p>Refactored 08.11.2002
 *
 * <p>&copy; 2002 Christian Treber, ct@ctreber.com
 * @author Christian Treber, ct@ctreber.com
 *
 */
public class DotGeneratorLines extends ScanDotGenerator
{
  private Projection fProjection;
  private int fLineDivider;
  private int fPointDivider;
  private static final double PI = Math.PI;
  private static final double TWOPI = 2 * PI;
  private static final double HALFPI = PI / 2;

  public DotGeneratorLines(Projection pProjection,
    int pLineDevider, int pPointDivider)
  {
    fProjection = pProjection;
    fLineDivider = pLineDevider;
    fPointDivider = pPointDivider;
  }

  /**
   * <p>Paint grid.
   */
  public void generateScanDots()
  {
    double lLonStep = TWOPI / (fLineDivider * 4);
    double lLatStep = PI / (fLineDivider * 2 * fPointDivider);
    for(double lLon = -PI; lLon <= PI; lLon += lLonStep)
    {
      for(double lLat = -HALFPI; lLat <= HALFPI; lLat += lLatStep)
      {
        transformAndAddDot(new Coordinate(lLat, lLon));
      }
    }

    lLatStep = TWOPI / (fLineDivider * 4);
    lLonStep = PI / (fLineDivider * 2 * fPointDivider);
    for(double lLat = -HALFPI; lLat <= HALFPI; lLat += lLatStep)
    {
      for(double lLon = -PI; lLon <= PI; lLon += lLonStep)
      {
        transformAndAddDot(new Coordinate(lLat, lLon));
      }
    }
  }

  private void transformAndAddDot(Coordinate pPos)
  {
    final Point3D lPointRotated = fProjection.rotate(pPos.getPoint3DRads());
    if(fProjection.isVisible(lPointRotated))
    {
      Point2D lPoint = fProjection.finalize(fProjection.project2D(lPointRotated));
      if(fProjection.isWithinImage(lPoint))
      {
        fDots.add(new ScanDot(ScanDot.DotTypeGrid, lPoint));
      }
    }
  }
}
