/* ========================================================================
 * PlantUML : a free UML diagram generator
 * ========================================================================
 *
 * (C) Copyright 2009-2020, Arnaud Roques
 *
 * Project Info:  https://plantuml.com
 * 
 * If you like this project or if you find it useful, you can support us at:
 * 
 * https://plantuml.com/patreon (only 1$ per month!)
 * https://plantuml.com/paypal
 * 
 * This file is part of PlantUML.
 *
 * PlantUML is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * PlantUML distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 *
 * Original Author:  Arnaud Roques
 */
package ext.plantuml.com.ctreber.acearth.scandot;

import java.util.Random;

/**
 * <p>Generate random stars as dots.
 *
 * <p>&copy; 2002 Christian Treber, ct@ctreber.com
 * @author Christian Treber, ct@ctreber.com
 *
 */
public class DotGeneratorStars extends ScanDotGenerator
{
  private final int fImageWidth;
  private final int fImageHeight;
  private int fBigStars;
  private double fStarFrequency;
  private final Random lRandom;

  public DotGeneratorStars(int pWidth, int pHeight,
    double pStarFrequency, int pBigStars, Random rnd)
  {
	lRandom = rnd;
    fImageWidth = pWidth;
    fImageHeight = pHeight;
    fStarFrequency = pStarFrequency;
    fBigStars = pBigStars;
  }

  public void generateScanDots()
  {
    // Make sure stars don't jump around between updates.
	    // final Random lRandom = new Random(ACearth.getStartTime());

    final int lStarsMax = (int)(fImageWidth * fImageHeight * fStarFrequency);
    for(int i = 0; i < lStarsMax; i++)
    {
      // "-1" to leave space for big stars.
      int x = (int)(lRandom.nextDouble() * (fImageWidth - 1));
      int y = (int)(lRandom.nextDouble() * fImageHeight);

      fDots.add(new ScanDot(ScanDot.DotTypeStar, x, y));

      // A big star is just two pixels wide.
      if((fBigStars != 0) && (Math.random() * 100 < fBigStars))
      {
        fDots.add(new ScanDot(ScanDot.DotTypeStar, x + 1, y));
      }
    }
  }
}
