/* ========================================================================
 * PlantUML : a free UML diagram generator
 * ========================================================================
 *
 * (C) Copyright 2009-2020, Arnaud Roques
 *
 * Project Info:  https://plantuml.com
 * 
 * If you like this project or if you find it useful, you can support us at:
 * 
 * https://plantuml.com/patreon (only 1$ per month!)
 * https://plantuml.com/paypal
 * 
 * This file is part of PlantUML.
 *
 * PlantUML is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * PlantUML distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 *
 * Original Author:  Arnaud Roques
 */
package ext.plantuml.com.ctreber.acearth.scandot;

import ext.plantuml.com.ctreber.acearth.util.Point2D;

/**
 * <p>A single scandot (opposed to a Polygon).
 *
 * <p>&copy; 2002 Christian Treber, ct@ctreber.com
 * @author Christian Treber, ct@ctreber.com
 *
 */
public class ScanDot implements Comparable
{
  // types of dots
  public static final int DotTypeStar = 0;
  public static final int DotTypeGrid = 1;

  private int fX;
  private int fY;
  private int fType;

  public ScanDot(int pType, int pX, int pY)
  {
    fType = pType;
    fX = pX;
    fY = pY;
  }

  public ScanDot(int pType, Point2D pPoint)
  {
    fType = pType;
    fX = (int)pPoint.getX();
    fY = (int)pPoint.getY();
  }

  public int compareTo(Object o)
  {
    if(o instanceof ScanDot)
    {
      ScanDot lOther = (ScanDot)o;

      return fY > lOther.fY ? 1 : (fY < lOther.fY ? -1 : 0);
    }

    throw new IllegalArgumentException("Can't compare to " + o.getClass());
  }

  public int getType()
  {
    return fType;
  }

  public int getX()
  {
    return fX;
  }

  public int getY()
  {
    return fY;
  }

  public String toString()
  {
    return fX + ", " + fY + ": " + fType;
  }
}
