/* ========================================================================
 * PlantUML : a free UML diagram generator
 * ========================================================================
 *
 * (C) Copyright 2009-2020, Arnaud Roques
 *
 * Project Info:  https://plantuml.com
 * 
 * If you like this project or if you find it useful, you can support us at:
 * 
 * https://plantuml.com/patreon (only 1$ per month!)
 * https://plantuml.com/paypal
 * 
 * This file is part of PlantUML.
 *
 * PlantUML is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * PlantUML distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 *
 * Original Author:  Arnaud Roques
 */
package ext.plantuml.com.ctreber.aclib.gui;

/**
 * <p></p>
 *
 * <p>&copy; 2002 Christian Treber, ct@ctreber.com</p>
 * @author Christian Treber, ct@ctreber.com
 *
 */
public class MODouble extends MonitoredObject
{
  private double fDouble;
  private boolean fCheckRange = false;
  private double fMin;
  private double fMax;

  public MODouble()
  {
  }

  public MODouble(double pDouble)
  {
    fDouble = pDouble;
  }

  public MODouble(double pDouble, double pMin, double pMax)
  {
    fMin = pMin;
    fMax = pMax;
    fCheckRange = true;
    set(pDouble);
  }

  public void set(double pDouble)
  {
    if(!checkRange(pDouble))
    {
      throw new IllegalArgumentException("Argument '" + pDouble +
        "' out of range [" + niceFormat(fMin) + "; " + niceFormat(fMax) + "]");
    }
    fDouble = pDouble;
    fireValueChanged();
  }

  private static String niceFormat(double pDouble)
  {
    if(pDouble == Double.MAX_VALUE)
    {
      return "Infinity";
    }

    if(pDouble == Double.MIN_VALUE)
    {
      return "-Infinity";
    }

    return Double.toString(pDouble);
  }

  public double get()
  {
    return fDouble;
  }

  private boolean checkRange(double pDouble)
  {
    return !fCheckRange || (fMin <= pDouble) && (pDouble <= fMax);
  }

  public boolean checkRange()
  {
    return checkRange(fDouble);
  }
}
