/* ========================================================================
 * PlantUML : a free UML diagram generator
 * ========================================================================
 *
 * (C) Copyright 2009-2020, Arnaud Roques
 *
 * Project Info:  https://plantuml.com
 * 
 * If you like this project or if you find it useful, you can support us at:
 * 
 * https://plantuml.com/patreon (only 1$ per month!)
 * https://plantuml.com/paypal
 * 
 * This file is part of PlantUML.
 *
 * PlantUML is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * PlantUML distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 *
 * Original Author:  Arnaud Roques
 */
package ext.plantuml.com.ctreber.aclib.gui;

import java.util.HashSet;

/**
 * <p>
 * Monitored enumeration value.
 * </p>
 * 
 * <p>
 * &copy; 2002 Christian Treber, ct@ctreber.com
 * </p>
 * 
 * @author Christian Treber, ct@ctreber.com
 * 
 */
public class MOEnum extends MonitoredObject {
	private HashSet fValidValues = new HashSet();
	/**
	 * <p>
	 * null if no value selected
	 */
	private Object fValue;

	public void addValidValue(Object pValue) {
		fValidValues.add(pValue);
	}

	public void set(Object pValue) {
		if (pValue != null) {
			checkValue(pValue);
		}

		fValue = pValue;
		fireValueChanged();
	}

	public Object get() {
		return fValue;
	}

	public boolean is(Object pObject) {
		checkValue(pObject);

		return this.equals(pObject);
	}

	public int hashCode() {
		if (fValue == null) {
			return 0;
		}

		return fValue.hashCode();
	}

	private void checkValue(Object pValue) {
		if (!fValidValues.contains(pValue)) {
			throw new IllegalArgumentException("Illegal enum value '" + pValue + "'");
		}
	}

	public boolean equals(Object obj) {
		if (obj instanceof MOEnum) {
			MOEnum lOther = (MOEnum) obj;
			if (fValue == null) {
				return lOther.fValue == null;
			}

			return fValue.equals(lOther.fValue);
		}

		if (fValue == null) {
			return obj.equals(null);
		}

		return fValue.equals(obj);
	}

	public HashSet getValidValues() {
		return fValidValues;
	}

	public boolean checkRange() {
		return true;
	}
}
