/* ========================================================================
 * PlantUML : a free UML diagram generator
 * ========================================================================
 *
 * (C) Copyright 2009-2020, Arnaud Roques
 *
 * Project Info:  https://plantuml.com
 * 
 * If you like this project or if you find it useful, you can support us at:
 * 
 * https://plantuml.com/patreon (only 1$ per month!)
 * https://plantuml.com/paypal
 * 
 * This file is part of PlantUML.
 *
 * PlantUML is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * PlantUML distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 *
 * Original Author:  Arnaud Roques
 */
package ext.plantuml.com.ctreber.aclib.gui;

/**
 * <p></p>
 *
 * <p>&copy; 2002 Christian Treber, ct@ctreber.com</p>
 * @author Christian Treber, ct@ctreber.com
 *
 */
public class MOInteger extends MonitoredObject
{
  private int fInteger;
  private boolean fCheckRange = false;
  private int fMin;
  private int fMax;

  public MOInteger()
  {
  }

  public MOInteger(int pInteger)
  {
    fInteger = pInteger;
  }

  public MOInteger(int pInteger, int pMin, int pMax)
  {
    fMin = pMin;
    fMax = pMax;
    fCheckRange = true;
    set(pInteger);
  }

  public void set(int pInteger)
  {
    if(!checkRange(pInteger))
    {
      throw new IllegalArgumentException("Argument '" + pInteger +
        "' out of range [" + niceFormat(fMin) + "; " + niceFormat(fMax) + "]");
    }
    fInteger = pInteger;
    fireValueChanged();
  }

  private static String niceFormat(int pInteger)
  {
    if(pInteger == Integer.MAX_VALUE)
    {
      return "Infinity";
    }

    if(pInteger == Integer.MIN_VALUE)
    {
      return "-Infinity";
    }

    return Integer.toString(pInteger);
  }

  public int get()
  {
    return fInteger;
  }

  private boolean checkRange(int pInteger)
  {
    return !fCheckRange || (fMin <= pInteger) && (pInteger <= fMax);
  }

  public boolean checkRange()
  {
    return checkRange(fInteger);
  }
}
