/* ========================================================================
 * PlantUML : a free UML diagram generator
 * ========================================================================
 *
 * (C) Copyright 2009-2020, Arnaud Roques
 *
 * Project Info:  https://plantuml.com
 * 
 * If you like this project or if you find it useful, you can support us at:
 * 
 * https://plantuml.com/patreon (only 1$ per month!)
 * https://plantuml.com/paypal
 * 
 * This file is part of PlantUML.
 *
 * PlantUML is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * PlantUML distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 *
 * Original Author:  Arnaud Roques
 */
/* 	This file is taken from
	https://github.com/andreas1327250/argon2-java

	Original Author: Andreas Gadermaier <up.gadermaier@gmail.com>
 */
package net.sourceforge.plantuml.argon2.model;

import static net.sourceforge.plantuml.argon2.Constants.ARGON2_SYNC_POINTS;

import net.sourceforge.plantuml.argon2.Argon2;

public class Instance {

    public Block[] memory;
    private int version;
    private int iterations;
    private int segmentLength;
    private int laneLength;
    private int lanes;

    private Argon2Type type;

    public Instance(Argon2 argon2) {
        this.version = argon2.getVersion();
        this.iterations = argon2.getIterations();
        this.lanes = argon2.getLanes();
        this.type = argon2.getType();

        /* 2. Align memory size */
        /* Minimum memoryBlocks = 8L blocks, where L is the number of lanes */
        int memoryBlocks = argon2.getMemory();

        if (memoryBlocks < 2 * ARGON2_SYNC_POINTS * argon2.getLanes()) {
            memoryBlocks = 2 * ARGON2_SYNC_POINTS * argon2.getLanes();
        }

        this.segmentLength = memoryBlocks / (argon2.getLanes() * ARGON2_SYNC_POINTS);
        this.laneLength = segmentLength * ARGON2_SYNC_POINTS;
        /* Ensure that all segments have equal length */
        memoryBlocks = segmentLength * (argon2.getLanes() * ARGON2_SYNC_POINTS);

        initMemory(memoryBlocks);
    }

    private void initMemory(int memoryBlocks) {
        this.memory = new Block[memoryBlocks];

        for (int i = 0; i < memory.length; i++) {
            memory[i] = new Block();
        }
    }

    public void clear() {
        for (Block b : memory) {
            b.clear();
        }

        memory = null;
    }

    public Block[] getMemory() {
        return memory;
    }

    public int getVersion() {
        return version;
    }

    public int getIterations() {
        return iterations;
    }

    public int getSegmentLength() {
        return segmentLength;
    }

    public int getLaneLength() {
        return laneLength;
    }

    public int getLanes() {
        return lanes;
    }

    public Argon2Type getType() {
        return type;
    }
}
