/* ========================================================================
 * PlantUML : a free UML diagram generator
 * ========================================================================
 *
 * (C) Copyright 2009-2020, Arnaud Roques
 *
 * Project Info:  https://plantuml.com
 * 
 * If you like this project or if you find it useful, you can support us at:
 * 
 * https://plantuml.com/patreon (only 1$ per month!)
 * https://plantuml.com/paypal
 * 
 * This file is part of PlantUML.
 *
 * PlantUML is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * PlantUML distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 *
 * Original Author:  Arnaud Roques
 */
package net.sourceforge.plantuml.dedication;

import java.math.BigInteger;

public class BlumBlumShub {

	private static final BigInteger two = BigInteger.valueOf(2L);

	private BigInteger state;
	private final BigInteger pq;

	public BlumBlumShub(BigInteger pq, byte[] seed) {
		this.pq = pq;
		this.state = new BigInteger(1, seed).mod(pq);
	}

	public int nextRnd(int numBits) {
		int result = 0;
		for (int i = numBits; i != 0; --i) {
			state = state.modPow(two, pq);
			final int bit = state.testBit(0) ? 1 : 0;
			result = (result << 1) | bit;
		}
		return result;
	}

}
