/* ========================================================================
 * PlantUML : a free UML diagram generator
 * ========================================================================
 *
 * (C) Copyright 2009-2020, Arnaud Roques
 *
 * Project Info:  https://plantuml.com
 * 
 * If you like this project or if you find it useful, you can support us at:
 * 
 * https://plantuml.com/patreon (only 1$ per month!)
 * https://plantuml.com/paypal
 * 
 * This file is part of PlantUML.
 *
 * PlantUML is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * PlantUML distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 *
 * Original Author:  Arnaud Roques
 */
package net.sourceforge.plantuml.ugraphic.comp;

public class Expand implements Comparable<Expand> {

	private final double position;
	private final double extend;
	private final ExpandType type;

	public Expand(ExpandType type, double position, double extend) {
		if (extend <= 0) {
			throw new IllegalArgumentException();
		}
		this.type = type;
		this.position = position;
		this.extend = extend;
	}

	@Override
	public String toString() {
		return "(" + position + "==>+" + extend + " " + type + ")";
	}

	public int compareTo(Expand other) {
		if (this.position < other.position) {
			return -1;
		}
		if (this.position > other.position) {
			return 1;
		}
		return 0;
	}

	public final double getPosition() {
		return position;
	}

	public final double getExtend() {
		return extend;
	}

	public final ExpandType getType() {
		return type;
	}

}
